from asyncio import iscoroutinefunction
from functools import wraps
from pydantic import BaseModel
from typing import Literal, Optional
from .exceptions import FunctionTypeError


class ValidateFields:
    """
    Field Validation Decorator
    """

    def __init__(
        self,
        mode: Literal['args', 'kwargs'] = 'kwargs',
        validate_model: Optional[str] = None,
        validate_model_index: Optional[int] = None,
        validate_function: str = 'validate_fields',
    ):
        """_summary_

        Args:
            mode (Literal[&#39;args&#39;, &#39;kwargs&#39;]): How to obtain the model that needs to be validated.
            validate_model (str, optional): The name of the pydantic model that needs to be validated in the function.
            validate_model_index (int, optional): The index of the pydantic model that needs to be validated in the function.
            validate_function (str, optional): The name of the validation function defined in the pydantic model. Defaults to 'validate_fields'.

        Raises:
            ValueError: The validate_model_index cannot be empty in args mode. || The validate_model cannot be empty in kwargs mode.
        """
        if mode == 'args' and validate_model_index is None:
            raise ValueError('The validate_model_index cannot be empty in args mode.')
        elif mode == 'kwargs' and validate_model is None:
            raise ValueError('The validate_model cannot be empty in kwargs mode.')
        self.mode = mode
        self.validate_model = validate_model
        self.validate_model_index = validate_model_index
        self.validate_function = validate_function

    def __call__(self, func):
        is_async = iscoroutinefunction(func)
        if is_async:

            @wraps(func)
            async def wrapper(*args, **kwargs):
                if self.mode == 'args':
                    validate_model = args[self.validate_model_index]
                else:
                    validate_model = kwargs.get(self.validate_model)
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.validate_function):
                    validate_function = getattr(
                        validate_model,
                        self.validate_function,
                    )
                    if callable(validate_function):
                        if iscoroutinefunction(validate_function):
                            await validate_function()
                        else:
                            validate_function()
                return await func(*args, **kwargs)

            return wrapper

        else:

            @wraps(func)
            def wrapper(*args, **kwargs):
                if self.mode == 'args':
                    validate_model = args[self.validate_model_index]
                else:
                    validate_model = kwargs.get(self.validate_model)
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.validate_function):
                    validate_function = getattr(
                        validate_model,
                        self.validate_function,
                    )
                    if callable(validate_function):
                        if iscoroutinefunction(validate_function):
                            raise FunctionTypeError(
                                error=f'The current function {func.__name__}() is a synchronous function. The function {self.validate_function}() is an asynchronous function and cannot be used in synchronous functions.',
                                category=RuntimeWarning,
                            )
                        validate_function()
                return func(*args, **kwargs)

            return wrapper



import asyncio
from pydantic_validation_decorator import (
    ValidateFields,
    Xss,
    FieldValidationError,
)
from pydantic import BaseModel
from typing import Optional


class XssTestModel(BaseModel):
    user_name: Optional[str] = None

    @Xss(
        field_name='user_name',
        message='user_name cannot contain script characters',
    )
    def get_user_name(self):
        return self.user_name

    def validate_fields(self):
        self.get_user_name()


@ValidateFields(validate_model='xss_test', validate_function='get_user_name')
def test_xss_decorator(xss_test: XssTestModel):
    return xss_test.model_dump()


def main():
    xss_test = XssTestModel(user_name='test123')
    try:
        print(test_xss_decorator(xss_test=xss_test))
    except FieldValidationError as e:
        print(e.__dict__)


@ValidateFields(mode='args', validate_model_index=0)
async def async_test_xss_decorator(
    xss_test: XssTestModel,
):
    return xss_test.model_dump()


async def async_main():
    xss_test = XssTestModel(user_name='test123<>')
    try:
        print(await async_test_xss_decorator(xss_test))
    except FieldValidationError as e:
        print(e.__dict__)


if __name__ == '__main__':
    main()
    asyncio.run(async_main())



from .validation import ValidateFields
from .network import Network
from .not_blank import NotBlank
from .pattern import Pattern
from .size import Size
from .xss import Xss
from .exceptions import FieldValidationError


__all__ = [
    'ValidateFields',
    'Network',
    'NotBlank',
    'Pattern',
    'Size',
    'Xss',
    'FieldValidationError',
]



import asyncio
from pydantic_validation_decorator import (
    ValidateFields,
    Size,
    FieldValidationError,
)
from pydantic import BaseModel
from typing import Optional


class SizeTestModel(BaseModel):
    dict_type: Optional[str] = None

    @Size(
        field_name='dict_type',
        min_length=0,
        max_length=10,
        message='The length of the dict_type cannot exceed 100 characters',
    )
    def get_dict_type(self):
        return self.dict_type

    def validate_fields(self):
        self.get_dict_type()


@ValidateFields(validate_model='size_test', validate_function='get_dict_type')
def test_size_decorator(size_test: SizeTestModel):
    return size_test.model_dump()


def main():
    size_test = SizeTestModel(dict_type='test')
    try:
        print(test_size_decorator(size_test=size_test))
    except FieldValidationError as e:
        print(e.__dict__)


@ValidateFields(mode='args', validate_model_index=0)
async def async_test_size_decorator(
    size_test: SizeTestModel,
):
    return size_test.model_dump()


async def async_main():
    size_test = SizeTestModel(dict_type='test_dict_type')
    try:
        print(await async_test_size_decorator(size_test))
    except FieldValidationError as e:
        print(e.__dict__)


if __name__ == '__main__':
    main()
    asyncio.run(async_main())



import asyncio
from pydantic_validation_decorator import (
    ValidateFields,
    Network,
    FieldValidationError,
)
from pydantic import BaseModel
from typing import Optional


class NetworkTestModel(BaseModel):
    email: Optional[str] = None

    @Network(
        field_name='email',
        field_type='EmailStr',
        message='email is invalid',
    )
    def get_email(self):
        return self.email

    def validate_fields(self):
        self.get_email()


@ValidateFields(validate_model='network_test', validate_function='get_email')
def test_network_decorator(network_test: NetworkTestModel):
    return network_test.model_dump()


def main():
    network_test = NetworkTestModel(email='test123@qq.com')
    try:
        print(test_network_decorator(network_test=network_test))
    except FieldValidationError as e:
        print(e.__dict__)
        

@ValidateFields(mode='args', validate_model_index=1)
async def async_test_network_decorator(
    test,
    network_test,
):
    return network_test.model_dump()


async def async_main():
    network_test = NetworkTestModel(email='test123')
    try:
        print(await async_test_network_decorator(1, network_test))
    except FieldValidationError as e:
        print(e.__dict__)


if __name__ == '__main__':
    main()
    asyncio.run(async_main())



from asyncio import iscoroutinefunction
from functools import wraps
from typing import Literal, Optional
from pydantic import (
    BaseModel,
    Field,
    AnyUrl,
    AnyHttpUrl,
    HttpUrl,
    AnyWebsocketUrl,
    WebsocketUrl,
    FileUrl,
    FtpUrl,
    PostgresDsn,
    CockroachDsn,
    AmqpDsn,
    RedisDsn,
    MongoDsn,
    KafkaDsn,
    NatsDsn,
    MySQLDsn,
    MariaDBDsn,
    ClickHouseDsn,
    EmailStr,
    NameEmail,
    IPvAnyAddress,
    ValidationError,
)
from .exceptions import FieldValidationError


class NetworkAnnotationModel(BaseModel):
    any_url: Optional[AnyUrl] = Field(default=None, description='Base type for all URLs.')
    any_http_url: Optional[AnyHttpUrl] = Field(
        default=None,
        description='A type that will accept any http or https URL.',
    )
    http_url: Optional[HttpUrl] = Field(
        default=None,
        description='A type that will accept any http or https URL and max length 2083.',
    )
    any_websocket_url: Optional[AnyWebsocketUrl] = Field(
        default=None,
        description='A type that will accept any ws or wss URL.',
    )
    websocket_url: Optional[WebsocketUrl] = Field(
        default=None,
        description='A type that will accept any ws or wss URL and max length 2083.',
    )
    file_url: Optional[FileUrl] = Field(default=None, description='A type that will accept any file URL.')
    ftp_url: Optional[FtpUrl] = Field(default=None, description='A type that will accept ftp URL.')
    postgres_dsn: Optional[PostgresDsn] = Field(
        default=None,
        description='A type that will accept any Postgres DSN.',
    )
    cockroach_dsn: Optional[CockroachDsn] = Field(
        default=None,
        description='A type that will accept any Cockroach DSN.',
    )
    amqp_dsn: Optional[AmqpDsn] = Field(default=None, description='A type that will accept any AMQP DSN.')
    redis_dsn: Optional[RedisDsn] = Field(default=None, description='A type that will accept any Redis DSN.')
    mongo_dsn: Optional[MongoDsn] = Field(
        default=None,
        description='A type that will accept any MongoDB DSN.',
    )
    kafka_dsn: Optional[KafkaDsn] = Field(default=None, description='A type that will accept any Kafka DSN.')
    nats_dsn: Optional[NatsDsn] = Field(default=None, description='A type that will accept any NATS DSN.')
    mysql_dsn: Optional[MySQLDsn] = Field(default=None, description='A type that will accept any MySQL DSN.')
    mariadb_dsn: Optional[MariaDBDsn] = Field(
        default=None,
        description='A type that will accept any MariaDB DSN.',
    )
    clickhouse_dsn: Optional[ClickHouseDsn] = Field(
        default=None,
        description='A type that will accept any ClickHouse DSN.',
    )
    email_str: Optional[EmailStr] = Field(default=None, description='Validate email addresses.')
    name_email: Optional[NameEmail] = Field(
        default=None,
        description='Validate a name and email address combination, as specified by RFC 5322.',
    )
    ipv_any_address: Optional[IPvAnyAddress] = Field(default=None, description='Validate an IPv4 or IPv6 address.')


class Network:
    """
    Field Network Type Validation Decorator
    """

    def __init__(
        self,
        field_name: str,
        field_type: Literal[
            'AnyUrl',
            'AnyHttpUrl',
            'HttpUrl',
            'AnyWebsocketUrl',
            'WebsocketUrl',
            'FileUrl',
            'FtpUrl',
            'PostgresDsn',
            'CockroachDsn',
            'AmqpDsn',
            'RedisDsn',
            'MongoDsn',
            'KafkaDsn',
            'NatsDsn',
            'MySQLDsn',
            'MariaDBDsn',
            'ClickHouseDsn',
            'EmailStr',
            'NameEmail',
            'IPvAnyAddress',
        ],
        message: Optional[str] = None,
    ):
        """Field Network Type Validation Decorator

        Args:
            field_name (str): Field name that need to be validate.
            field_type (Literal[ &#39;AnyUrl&#39;, &#39;AnyHttpUrl&#39;, &#39;HttpUrl&#39;, &#39;AnyWebsocketUrl&#39;, &#39;WebsocketUrl&#39;, &#39;FileUrl&#39;, &#39;FtpUrl&#39;, &#39;PostgresDsn&#39;, &#39;CockroachDsn&#39;, &#39;AmqpDsn&#39;, &#39;RedisDsn&#39;, &#39;MongoDsn&#39;, &#39;KafkaDsn&#39;, &#39;NatsDsn&#39;, &#39;MySQLDsn&#39;, &#39;MariaDBDsn&#39;, &#39;ClickHouseDsn&#39;, &#39;EmailStr&#39;, &#39;NameEmail&#39;, &#39;IPvAnyAddress&#39;, ]): Field type that need to be validate.
            message (Optional[str], optional): Prompt message for validation failure. Defaults to None.
        """
        self.field_name = field_name
        self.field_type = field_type
        self.message = message

    def __call__(self, func):
        is_async = iscoroutinefunction(func)
        if is_async:

            @wraps(func)
            async def wrapper(*args, **kwargs):
                validate_model = args[0]
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.field_name):
                    field_value = getattr(validate_model, self.field_name)
                    if field_value:
                        try:
                            if self.field_type == 'AnyUrl':
                                NetworkAnnotationModel(any_url=field_value)
                            elif self.field_type == 'AnyHttpUrl':
                                NetworkAnnotationModel(any_http_url=field_value)
                            elif self.field_type == 'HttpUrl':
                                NetworkAnnotationModel(http_url=field_value)
                            elif self.field_type == 'AnyWebsocketUrl':
                                NetworkAnnotationModel(any_websocket_url=field_value)
                            elif self.field_type == 'WebsocketUrl':
                                NetworkAnnotationModel(websocket_url=field_value)
                            elif self.field_type == 'FileUrl':
                                NetworkAnnotationModel(file_url=field_value)
                            elif self.field_type == 'FtpUrl':
                                NetworkAnnotationModel(ftp_url=field_value)
                            elif self.field_type == 'PostgresDsn':
                                NetworkAnnotationModel(postgres_dsn=field_value)
                            elif self.field_type == 'CockroachDsn':
                                NetworkAnnotationModel(cockroach_dsn=field_value)
                            elif self.field_type == 'AmqpDsn':
                                NetworkAnnotationModel(amqp_dsn=field_value)
                            elif self.field_type == 'RedisDsn':
                                NetworkAnnotationModel(redis_dsn=field_value)
                            elif self.field_type == 'MongoDsn':
                                NetworkAnnotationModel(mongo_dsn=field_value)
                            elif self.field_type == 'KafkaDsn':
                                NetworkAnnotationModel(kafka_dsn=field_value)
                            elif self.field_type == 'NatsDsn':
                                NetworkAnnotationModel(nats_dsn=field_value)
                            elif self.field_type == 'MySQLDsn':
                                NetworkAnnotationModel(mysql_dsn=field_value)
                            elif self.field_type == 'MariaDBDsn':
                                NetworkAnnotationModel(mariadb_dsn=field_value)
                            elif self.field_type == 'ClickHouseDsn':
                                NetworkAnnotationModel(clickhouse_dsn=field_value)
                            elif self.field_type == 'EmailStr':
                                NetworkAnnotationModel(email_str=field_value)
                            elif self.field_type == 'NameEmail':
                                NetworkAnnotationModel(name_email=field_value)
                            elif self.field_type == 'IPvAnyAddress':
                                NetworkAnnotationModel(ipv_any_address=field_value)
                        except (
                            ValidationError,
                            ValueError,
                        ):
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} is not the correct {self.field_type} type.',
                            )
                return await func(*args, **kwargs)

            return wrapper

        else:

            @wraps(func)
            def wrapper(*args, **kwargs):
                validate_model = args[0]
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.field_name):
                    field_value = getattr(validate_model, self.field_name)
                    if field_value:
                        try:
                            if self.field_type == 'AnyUrl':
                                NetworkAnnotationModel(any_url=field_value)
                            elif self.field_type == 'AnyHttpUrl':
                                NetworkAnnotationModel(any_http_url=field_value)
                            elif self.field_type == 'HttpUrl':
                                NetworkAnnotationModel(http_url=field_value)
                            elif self.field_type == 'AnyWebsocketUrl':
                                NetworkAnnotationModel(any_websocket_url=field_value)
                            elif self.field_type == 'WebsocketUrl':
                                NetworkAnnotationModel(websocket_url=field_value)
                            elif self.field_type == 'FileUrl':
                                NetworkAnnotationModel(file_url=field_value)
                            elif self.field_type == 'FtpUrl':
                                NetworkAnnotationModel(ftp_url=field_value)
                            elif self.field_type == 'PostgresDsn':
                                NetworkAnnotationModel(postgres_dsn=field_value)
                            elif self.field_type == 'CockroachDsn':
                                NetworkAnnotationModel(cockroach_dsn=field_value)
                            elif self.field_type == 'AmqpDsn':
                                NetworkAnnotationModel(amqp_dsn=field_value)
                            elif self.field_type == 'RedisDsn':
                                NetworkAnnotationModel(redis_dsn=field_value)
                            elif self.field_type == 'MongoDsn':
                                NetworkAnnotationModel(mongo_dsn=field_value)
                            elif self.field_type == 'KafkaDsn':
                                NetworkAnnotationModel(kafka_dsn=field_value)
                            elif self.field_type == 'NatsDsn':
                                NetworkAnnotationModel(nats_dsn=field_value)
                            elif self.field_type == 'MySQLDsn':
                                NetworkAnnotationModel(mysql_dsn=field_value)
                            elif self.field_type == 'MariaDBDsn':
                                NetworkAnnotationModel(mariadb_dsn=field_value)
                            elif self.field_type == 'ClickHouseDsn':
                                NetworkAnnotationModel(clickhouse_dsn=field_value)
                            elif self.field_type == 'EmailStr':
                                NetworkAnnotationModel(email_str=field_value)
                            elif self.field_type == 'NameEmail':
                                NetworkAnnotationModel(name_email=field_value)
                            elif self.field_type == 'IPvAnyAddress':
                                NetworkAnnotationModel(ipv_any_address=field_value)
                        except (
                            ValidationError,
                            ValueError,
                        ):
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} is not the correct {self.field_type} type.',
                            )
                return func(*args, **kwargs)

            return wrapper



from typing import Any
from warnings import warn


class FieldValidationError(Exception):
    """
    Custom Field Validation Exception FieldValidationError
    """

    def __init__(
        self,
        model_name: str = None,
        field_name: str = None,
        field_value: Any = None,
        validator: str = None,
        message: str = None,
    ):
        """Custom Field Validation Exception FieldValidationError

        Args:
            model_name (str, optional): Model name with error. Defaults to None.
            field_name (str, optional): Field name with error. Defaults to None.
            field_value (Any, optional): Field value with errors. Defaults to None.
            validator (str, optional): Validation decorator with errors. Defaults to None.
            message (str, optional): Prompt message for validation failure. Defaults to None.
        """
        self.model_name = model_name
        self.field_name = field_name
        self.field_value = field_value
        self.validator = validator
        self.message = message


class FunctionTypeError(Exception):
    """
    Custom Function Type Exception FunctionTypeError
    """

    def __init__(self, error: str = None, category: Any = None):
        """Custom Function Type Exception FunctionTypeError

        Args:
            error (str, optional): Error message. Defaults to None.
            category (Any, optional): Warning category. Defaults to None.
        """
        self.error = error
        self.category = category
        warn(self.error, category=self.category)



from asyncio import iscoroutinefunction
from functools import wraps
from pydantic import BaseModel
from typing import Optional
from .exceptions import FieldValidationError


class NotBlank:
    """
    Field NotBlank Validation Decorator
    """

    def __init__(
        self,
        field_name: str,
        message: Optional[str] = None,
    ):
        """Field NotBlank Validation Decorator

        Args:
            field_name (str): Field name that need to be validate.
            message (Optional[str], optional): Prompt message for validation failure. Defaults to None.
        """
        self.field_name = field_name
        self.message = message

    def __call__(self, func):
        is_async = iscoroutinefunction(func)
        if is_async:

            @wraps(func)
            async def wrapper(*args, **kwargs):
                validate_model = args[0]
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.field_name):
                    field_value = getattr(validate_model, self.field_name)
                    if (
                        field_value is None
                        or field_value == ''
                        or field_value == []
                        or field_value == ()
                        or field_value == {}
                    ):
                        raise FieldValidationError(
                            model_name=validate_model.__class__.__name__,
                            field_name=self.field_name,
                            field_value=field_value,
                            validator=self.__class__.__name__,
                            message=self.message if self.message else f'{self.field_name} cannot be empty.',
                        )
                return await func(*args, **kwargs)
        else:

            @wraps(func)
            def wrapper(*args, **kwargs):
                validate_model = args[0]
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.field_name):
                    field_value = getattr(validate_model, self.field_name)
                    if (
                        field_value is None
                        or field_value == ''
                        or field_value == []
                        or field_value == ()
                        or field_value == {}
                    ):
                        raise FieldValidationError(
                            model_name=validate_model.__class__.__name__,
                            field_name=self.field_name,
                            field_value=field_value,
                            validator=self.__class__.__name__,
                            message=self.message if self.message else f'{self.field_name} cannot be empty.',
                        )
                return func(*args, **kwargs)

            return wrapper



import asyncio
from pydantic_validation_decorator import (
    ValidateFields,
    Pattern,
    FieldValidationError,
)
from pydantic import BaseModel
from typing import Optional


class PatternTestModel(BaseModel):
    dict_type: Optional[str] = None

    @Pattern(
        field_name='dict_type',
        regexp='^[a-z][a-z0-9_]*$',
        message='The dict_type must start with a letter and can only be lowercase letters, numbers, and dashes',
    )
    def get_dict_type(self):
        return self.dict_type

    def validate_fields(self):
        self.get_dict_type()


@ValidateFields(validate_model='pattern_test', validate_function='get_dict_type')
def test_pattern_decorator(pattern_test: PatternTestModel):
    return pattern_test.model_dump()


def main():
    pattern_test = PatternTestModel(dict_type='test_dict_type')
    try:
        print(test_pattern_decorator(pattern_test=pattern_test))
    except FieldValidationError as e:
        print(e.__dict__)


@ValidateFields(mode='args', validate_model_index=0)
async def async_test_pattern_decorator(
    pattern_test: PatternTestModel,
):
    return pattern_test.model_dump()


async def async_main():
    pattern_test = PatternTestModel(dict_type='123')
    try:
        print(await async_test_pattern_decorator(pattern_test))
    except FieldValidationError as e:
        print(e.__dict__)


if __name__ == '__main__':
    main()
    asyncio.run(async_main())



import io
from setuptools import setup, find_packages
from pkg_resources import parse_requirements


with open('requirements.txt', encoding='utf-8') as fp:
    install_requires = [str(requirement) for requirement in parse_requirements(fp)]

setup(
    name='pydantic_validation_decorator',
    version='0.1.2',
    author_email='3055204202@qq.com',
    homepage='https://github.com/insistence/pydantic-validation-decorator',
    author='insistence <3055204202@qq.com>',
    packages=find_packages(),
    license='MIT',
    description='Some practical pydantic validation decorators that support manual invocation.',
    long_description=io.open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    python_requires='>=3.8',
    classifiers=[
        'Framework :: Pydantic',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    url='https://github.com/insistence/pydantic-validation-decorator',
    install_requires=install_requires,
    include_package_data=True,
)



from asyncio import iscoroutinefunction
from functools import wraps
from typing import Optional
from pydantic import BaseModel
from .exceptions import FieldValidationError


class Size:
    """
    Field Size Validation Decorator
    """

    def __init__(
        self,
        field_name: str,
        gt: Optional[float] = None,
        ge: Optional[float] = None,
        lt: Optional[float] = None,
        le: Optional[float] = None,
        min_length: Optional[int] = 0,
        max_length: Optional[int] = None,
        message: Optional[str] = None,
    ):
        """Field Size Validation Decorator

        Args:
            field_name (str): Field name that need to be validate.
            gt (Optional[float], optional): The numerical field value must be greater than gt. Defaults to None.
            ge (Optional[float], optional): The numerical field value must be greater than or equal to ge. Defaults to None.
            lt (Optional[float], optional): The numerical field value must be less than lt. Defaults to None.
            le (Optional[float], optional): The numerical field value must be less than or equal to le. Defaults to None.
            min_length (Optional[int], optional): The length of a string field cannot be less than min_length. Defaults to 0.
            max_length (Optional[int], optional): The length of a string field cannot be greater than max_length. Defaults to None.
            message (Optional[str], optional): Prompt message for validation failure. Defaults to None.
        """
        self.field_name = field_name
        self.gt = gt
        self.ge = ge
        self.lt = lt
        self.le = le
        self.min_length = min_length if min_length >= 0 else 0
        self.max_length = max_length
        self.message = message

    def __call__(self, func):
        is_async = iscoroutinefunction(func)
        if is_async:

            @wraps(func)
            async def wrapper(*args, **kwargs):
                validate_model = args[0]
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.field_name):
                    field_value = getattr(validate_model, self.field_name)
                    if isinstance(field_value, float):
                        if self.gt is not None and field_value <= self.gt:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be greater than {self.gt}.',
                            )
                        elif self.ge is not None and field_value < self.ge:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be greater than or equal to {self.ge}.',
                            )
                        elif self.lt is not None and field_value >= self.lt:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be less than {self.lt}.',
                            )
                        elif self.le is not None and field_value > self.le:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be less than or equal to {self.le}.',
                            )
                    elif isinstance(field_value, str):
                        if len(field_value) < self.min_length:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'The length of {self.field_name} cannot be less than {self.min_length}.',
                            )
                        elif self.max_length is not None and len(field_value) > self.max_length:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'The length of {self.field_name} cannot be greater than {self.max_length}.',
                            )
                return await func(*args, **kwargs)

            return wrapper

        else:

            @wraps(func)
            def wrapper(*args, **kwargs):
                validate_model = args[0]
                if isinstance(validate_model, BaseModel) and hasattr(validate_model, self.field_name):
                    field_value = getattr(validate_model, self.field_name)
                    if isinstance(field_value, float):
                        if self.gt is not None and field_value <= self.gt:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be greater than {self.gt}.',
                            )
                        elif self.ge is not None and field_value < self.ge:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be greater than or equal to {self.ge}.',
                            )
                        elif self.lt is not None and field_value >= self.lt:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be less than {self.lt}.',
                            )
                        elif self.le is not None and field_value > self.le:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'{self.field_name} must be less than or equal to {self.le}.',
                            )
                    elif isinstance(field_value, str):
                        if len(field_value) < self.min_length:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'The length of {self.field_name} cannot be less than {self.min_length}.',
                            )
                        elif self.max_length is not None and len(field_value) > self.max_length:
                            raise FieldValidationError(
                                model_name=validate_model.__class__.__name__,
                                field_name=self.field_name,
                                field_value=field_value,
                                validator=self.__class__.__name__,
                                message=self.message
                                if self.message
                                else f'The length of {self.field_name} cannot be greater than {self.max_length}.',
                            )
                return func(*args, **kwargs)

            return wrapper



class StringUtils:
    """
    String Utils Class
    """

    @classmethod
    def is_blank(cls, string: str) -> bool:
        """Validate if the string is '' or all spaces

        Args:
            string (str): String to be validated

        Returns:
            bool: Validation results
        """
        if string is None:
            return False
        str_len = len(string)
        if str_len == 0:
            return True
        else:
            for i in range(str_len):
                if string[i] != ' ':
                    return False
            return True

    @classmethod
    def is_empty(cls, string: str) -> bool:
        """Validate if the string is '' or None

        Args:
            string (str): String to be validated

        Returns:
            bool: Validation results
        """
        return string is None or len(string) == 0
